#!/usr/bin/perl -w

#
# crp_conf.pm
# Copyright (C) 2006 by John Heidemann  <johnh@isi.edu>
# $Id: crp_conf.pl 97 2006-04-14 00:27:27Z johnh $
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

package crp;

#
# This is a simple front-end for perl access to the CRP database.
#

use strict;
use warnings;

BEGIN {
    use Exporter ();
    our (@ISA, @EXPORT, @EXPORT_OK);

     @ISA         = qw(Exporter);
     @EXPORT      = qw();
     @EXPORT_OK   = qw(connect_database email_to_contactId qe
		    %conf);
}

# read the config file
our %conf;
sub read_conf {
    open(IN,"<crp.conf") or die "cannot read crp.conf\n";
    while (<IN>) {
	next if (/^\s*\#/);  # comments
	next if (/^\s*$/);   # blank linkes
	my ($key, $value) = /\s*(\S+)\s*=\s*(\S+)\s*$/;
	die "cannot parse crp.conf line $_\n" unless (defined($key) && defined($value));
	$conf{$key} = $value;
    };
    close IN;
}

sub connect_database {
    my($dbh) = DBI->connect("DBI:mysql:" . $conf{'DB_NAME'},
	$conf{'DB_USER'}, $conf{'DB_PASSWORD'})
	or die "cannot contact database.\n";
    return $dbh;
}

sub qe {
    my($dbh) = shift @_;
    die "bad dbh.\n" if (!ref($dbh));
    my($sql) = shift @_;
    my $qhand = $dbh->prepare($sql) or die "cannot prepare: $sql\n";
    $qhand->execute(@_) or die "cannot execute: $sql\n";
    return $qhand;
}

sub email_to_contactId {
    my($dbh, $who) = @_;
    die "email_to_contactId: empy user.\n" if (!defined($who) || $who eq '');
    my $quser = qe($dbh, "SELECT contactId FROM ContactInfo WHERE EMAIL=?",
		$who);
    my $user_aref = $quser->fetchall_arrayref;
    die "no user $who found\n" if ($#{$user_aref} < 0);
    die "multiple users $who found\n" if ($#{$user_aref} > 0);
    return $user_aref->[0][0];
}

sub log_about_paper {
    my($dbh, $paper_id, $contact_id, $text) = @_;
    qe($dbh, "INSERT INTO ActionLog SET ipaddr=?, contactId=?, paperId=?, action=?",
	"direct-db", $contact_id, $paper_id, $text);

}

&read_conf;
1;
